# revzones
# Compute reverse names for IP networks and hosts

from ipaddr import IPv4Network, IPv6Network, IPAddress


def reverse_name(ipnet, ipa=None):

    def _get_element(packed, index):
        val = packed[index // 8]
        shift = 8 - (index % 8 + step)
        val = (val >> shift) & ((1 << step) - 1)
        return val

    if ipnet.version == 6:
        revn = "ip6.arpa"
        step = 4
        base = 'x'
        addrlen = 128
    else:
        revn = "in-addr.arpa"
        step = 8
        base = 'd'
        addrlen = 32
    prefixlen = ipnet.prefixlen
    index = 0

    # Generate network part

    while index < prefixlen:
        val = _get_element(ipnet.packed, index)
        if prefixlen - index < step:
            suffix = "-" + str(prefixlen)
        else:
            suffix = ""
        revn = format(val, base) + suffix + "." + revn
        index = index + step

    # Generate host part

    if ipa:
        index = prefixlen - prefixlen % step
        while index < addrlen:
            val = _get_element(ipa.packed, index)
            revn = format(val, base) + "." + revn
            index = index + step

    return revn

if __name__ == '__main__':
    for (ipnet, ipa) in [
        (IPv4Network('192.168.12.128/26'), '192.168.12.154'),
        (IPv4Network('192.168.12.128/24'), '192.168.12.154'),
        (IPv4Network('192.168.12.128/18'), '192.168.12.154'),
        (IPv4Network('192.168.12.128/16'), '192.168.12.154'),
        (IPv6Network('2001:470:1f0b:1b0c::/64'),
            '2001:470:1f0b:1b0c::1234:5678'),
        (IPv6Network('2001:470:1f0b:1b0c::/62'),
            '2001:470:1f0b:1b0c::1234:5678'),
        (IPv6Network('2001:470:1f0b:1b0c::/48'),
            '2001:470:1f0b:1b0c::1234:5678')]:
        print("Net %s -> %s" % (str(ipnet), reverse_name(ipnet, None)))
        print("Host %s -> %s" % (ipa, reverse_name(ipnet, IPAddress(ipa))))
